<?php

/*
 * Plugin Alternate-Scoretable
 * This plugin is an alternative Scoretable, it´s looks nicer (that´s my opinion) and
 * shows more nicer information of each Player:
 *  - Player Position
 *  - Player Avatar
 *  - Player Nickname
 *  - The current best time/score for this Track
 *  - The personal best time/score for this Track
 *  - Total-Score at this Server and this Session
 *  - Amount how often this Track was finished or other related counts/scores (depends on GameMode)
 *  - The current Ladder-Rank
 *  - The current Ladder-Score
 *  - The Online-Time since connect to this Server
 *
 * Just take a look at the Screenshots at http://www.tm-forum.com/viewtopic.php?f=127&t=26138
 *
 * This Plugin works only with TMF and all the Gamemodes: Rounds, Team, TimeAttack,
 * Stunts, Laps and Cup. The Layout of the Scoretable in Team-Mode is just a little
 * bit different as the other one.
 *
 * Hardcoded limit of Players in Scoretable in the Gamemodes Rounds, TimeAttack, Stunts,
 * Laps and Cup are 300 and 40 Players in Gamemode Team (20 Players for each Team).
 *
 * Known errors or bad behavior:
 *  - The Total Ladder-Score are display delayed for one Track, because the 'LadderScore'
 *    in the ListMethod GetCurrentRanking() from the dedicated Server are not refreshed
 *    within the Score. It takes some time the 'LadderScore' is refreshed.
 *  - Fix the displaying of avatar of the player himself, currently are not displaying,
 *    other Player-Avatars are displayed.
 *  - Click of "Server to Favorites" and "Add to Buddylist" in the PlayerWidget results
 *    that the Player leaves the Server, because it´s inplemented with the TMTP-Protocol.
 *
 * ----------------------------------------------------------------------------------
 * Author:	undef.de
 * Version:	0.8.7
 * Date:	2010-11-12
 * Copyright:	2009 - 2010 by undef.de
 * Home:	http://www.undef.de/Trackmania/Plugins/
 * System:	xaseco/1.12 and up
 * ----------------------------------------------------------------------------------
 *
 * LICENSE: This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * ----------------------------------------------------------------------------------
 *
 * The Bar-Images included in this Plugin are licensed by the Author under a
 * Creative Commons Attribution-Share Alike 3.0 Germany License.
 * See the following links for details:
 * German:	http://creativecommons.org/licenses/by-sa/3.0/de/legalcode
 * English:	http://creativecommons.org/licenses/by-sa/3.0/legalcode
 *
 * ----------------------------------------------------------------------------------
 *
 * Dependencies:	plugins/plugin.panels.php
 */

/* The following manialink id's are used in this plugin (the 916 part of id can be changed on trouble):
 * 91600		id for manialinks
 * 91601		id for manialink Scoretable
 * 91602		id for action display Scoretable
 * 91603		id for action close Scoretable
 * 91604		id for manialink Legend
 * 91605		id for action display Legend
 * 91606		id for action close Legend
 * 91607		id for manialink of Quicklink-Button
 * 91608		id for manialink of PlayerWidget
 * 91609		id for action close PlayerWidget
 * 91610		id for (reserved for later use)
 * 91620-91635		id for action previous page in Scoretable (max. 15 pages)
 * 91640-91655		id for action next page in Scoretable (max. 15 pages)
 * 91660-9169999999	id for identify selected player for PlayerWidget
 *
 */

Aseco::registerEvent('onSync',				'ast_onSync');
Aseco::registerEvent('onPlayerConnect',			'ast_onPlayerConnect');
Aseco::registerEvent('onPlayerFinish',			'ast_onPlayerFinish');
Aseco::registerEvent('onPlayerInfoChanged',		'ast_onPlayerInfoChanged');
Aseco::registerEvent('onPlayerDisconnect',		'ast_onPlayerDisconnect');
Aseco::registerEvent('onPlayerManialinkPageAnswer',	'ast_onPlayerManialinkPageAnswer');
Aseco::registerEvent('onNewChallenge',			'ast_onNewChallenge');			// throw main 'begin challenge' event
Aseco::registerEvent('onRestartChallenge2',		'ast_onRestartChallenge2');		// throw 'restarted challenge' event
Aseco::registerEvent("onBeginRound", 			'ast_onBeginRound');
Aseco::registerEvent("onEndRound", 			'ast_onEndRound');
Aseco::registerEvent('onEndRace1',			'ast_onEndRace1');			// throw pre event for chat-based votes
Aseco::registerEvent('onEndRace',			'ast_onEndRace');			// throw main 'end challenge' event
Aseco::registerEvent('onEverySecond',			'ast_onEverySecond');

Aseco::addChatCommand('scoretable',			'Display the alternate Scoretable (see: /scoretable help)');

global $ast_config, $ast_todo;
$ast_todo = array();

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onSync
function ast_onSync ($aseco) {
	global $ast_config;

	// Check for the right XASECO-Version
	if ( defined('XASECO_VERSION') ) {
		if ( version_compare(XASECO_VERSION, 1.12, '<') ) {
			trigger_error('[plugin.alternate_scoretable.php] Not supported xaseco version ('. XASECO_VERSION .')! Please update to min. version 1.12!', E_USER_ERROR);
		}
	}
	else {
		trigger_error('[plugin.alternate_scoretable.php] Can not identify the System, "XASECO_VERSION" is unset! This plugin runs only with XASECO/1.12 and up.', E_USER_ERROR);
	}


	// Check for dependencies
	if ( !function_exists('panels_default') ) {
		trigger_error('[plugin.alternate_scoretable.php] Missing dependent plugin, please activate "plugin.panels.php" in "plugins.xml" and restart.', E_USER_ERROR);
	}

	if ($aseco->server->getGame() != 'TMF') {
		trigger_error('[plugin.alternate_scoretable.php] This plugin supports only TMF, can not start with a '. $aseco->server->getGame() .' Dedicated-Server!', E_USER_ERROR);
	}


	if (!$ast_config = $aseco->xml_parser->parseXML('alternate_scoretable.xml')) {
		trigger_error('[plugin.alternate_scoretable.php] Could not read/parse config file "alternate_scoretable.xml"!', E_USER_ERROR);
	}

	$ast_config = $ast_config['ALTERNATE_SCORETABLE'];


	$ast_config['MANIALINK_ID'] = '916';
	$ast_config['VERSION'] = '0.8.7';
	$ast_config['PLAYERS'] = array();


	// Set current State
	$ast_config['CURRENT_STATE'] = 1;

	// Set current Gamemode
	$ast_config['CURRENT_GAMEMODE'] = $aseco->server->gameinfo->mode;

	// Show the Quicklink button?
	if (strtoupper($ast_config['QUICKLINK_BUTTON'][0]['ENABLED'][0]) == 'TRUE') {
		// $aseco, $caller, $display
		ast_widgetQuicklink($aseco, false, true);
	}

	// Set Score-Display-Time
	if ($ast_config['SCORE_DISPLAY_TIME'][0] < 15) {
		$ast_config['SCORE_DISPLAY_TIME'][0] = 15;
	}
	$aseco->client->query('SetChatTime', (intval($ast_config['SCORE_DISPLAY_TIME'][0] / 1.3333) * 1000));		// Milliseconds



	// Remove the function scorepanel_on() from manialinks.inc.php at the event 'onEndRace'
	// to prevent displaying Automatic Scoretable instead Alternate Scoretable
	$array_pos = 0;
	foreach ($aseco->events['onEndRace'] as $func_name) {
		if ($func_name == 'scorepanel_on') {
			unset($aseco->events['onEndRace'][$array_pos]);
			break;
		}
		$array_pos ++;
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onEverySecond
function ast_onEverySecond ($aseco) {
	global $ast_config, $ast_todo;


	// Do Player based delayed calls
	foreach ($ast_config['PLAYERS'] as $login => $struct) {
		if ($ast_config['PLAYERS']{$login}->delayed_call != false) {

			// Start the job if wait is over
			if (time() >= $ast_config['PLAYERS']{$login}->delayed_call['delay']) {
				eval($ast_config['PLAYERS']{$login}->delayed_call['func']);
				$ast_config['PLAYERS']{$login}->delayed_call = false;
			}
		}
	}

	// Bail out if nothing to do for players(global) delayed calls
	if (count($ast_todo) == 0) {
		return;
	}

	// Start the jobs at given times
	foreach ($ast_todo as &$job) {

		// Skip job if already done
		if ($job['delay'] == -1) {
			continue;
		}

		// Start the job if wait is over
		if (time() >= $job['delay']) {
			eval($job['func']);
			$job['delay'] = -1;
			$job['func'] = false;
		}
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function chat_scoretable ($aseco, $command) {
	global $ast_config;


	// Check optional parameter
	if (strtoupper($command['params']) == 'HELP') {

		ast_widgetLegend($aseco, $command['author'], true);

	}
	else {

		// $aseco, $caller, $timeout, $display_close, $page
		ast_showScoretable($aseco, $command['author']->login, 0, true, 0);

	}


	// Show message
	if ( isset($message) ) {
		if ( (strtoupper($ast_config['MESSAGES_IN_WINDOW'][0]) == 'TRUE') && (function_exists('send_window_message')) ) {
			send_window_message($aseco, $message, $command['author']);
		}
		else {
			$aseco->client->query('ChatSendServerMessageToLogin', $aseco->formatColors($message), $command['author']->login);
		}
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function ast_showScoretable ($aseco, $caller = false, $timeout = 0, $display_close = true, $page = 0) {
	global $ast_config;


	// Refresh online time and Personal Best of each (connected) Player
	foreach ($aseco->server->players->player_list as $player) {

		// Get actual connect time
		$ast_config['PLAYERS']{$player->login}->online_since = formatTimeH($player->getTimeOnline() * 1000, false);

		// Get personal best driven time for current map
		$ast_config['PLAYERS']{$player->login}->personal_best = $player->panels['pb'];

		// Get Spectator status
		$ast_config['PLAYERS']{$player->login}->isspectator = $player->isspectator;
	}


	$xml =  '<?xml version="1.0" encoding="UTF-8"?>';
	$xml .= '<manialinks id="'. $ast_config['MANIALINK_ID'] .'00">';
	$xml .= '<manialink id="'. $ast_config['MANIALINK_ID'] .'01">';

	// Window
	$xml .= '<frame posn="-40.1 30.45 -3">';	// BEGIN: Window Frame
	$xml .= '<quad posn="0.8 -0.8 0.10" sizen="78.4 53.7" bgcolor="000A"/>';
	$xml .= '<quad posn="-0.2 0.2 0.11" sizen="80.4 55.7" style="Bgs1InRace" substyle="BgCard3"/>';

	// Header Icon
	$xml .= '<quad posn="-1.7 2 0.20" sizen="8 5" style="Icons64x64_1" substyle="RestartRace"/>';

	// Link and About
	$xml .= '<quad posn="52 -54.1 0.12" sizen="14.5 1" url="http://www.undef.de/Trackmania/Plugins/" bgcolor="0000"/>';
	$xml .= '<label posn="52 -54.1 0.12" sizen="30 1" halign="left" textsize="1" scale="0.7" textcolor="000F" text="ALTERNATE SCORETABLE/'. $ast_config['VERSION'] .'"/>';

	if ($display_close == true) {
		// Close Button
		$xml .= '<frame posn="77.4 1.3 0">';
		$xml .= '<quad posn="0 0 0.12" sizen="4 4" style="Icons64x64_1" substyle="ArrowDown"/>';
		$xml .= '<quad posn="1.1 -1.35 0.13" sizen="1.8 1.75" bgcolor="EEEF"/>';
		$xml .= '<quad posn="0.65 -0.7 0.14" sizen="2.6 2.6" action="'. $ast_config['MANIALINK_ID'] .'03" style="Icons64x64_1" substyle="Close"/>';
		$xml .= '</frame>';
	}



	// Frame for Legend/Previous/Next Buttons
	$xml .= '<frame posn="67.05 -53.2 0">';

	// Legend button
	$xml .= '<frame posn="0 0 0">';
	$xml .= '<quad posn="0 0 0.12" sizen="3.2 3.2" action="'. $ast_config['MANIALINK_ID'] .'05" style="Icons64x64_1" substyle="ArrowDown"/>';
	$xml .= '<quad posn="0.85 -1.1 0.13" sizen="1.5 1.4" bgcolor="EEEF"/>';
	$xml .= '<quad posn="0.65 -0.7 0.14" sizen="1.8 1.8" style="Icons64x64_1" substyle="TrackInfo"/>';
	$xml .= '</frame>';

	// Display Previous/Next-Buttons not in Team-Mode
	if ($ast_config['CURRENT_GAMEMODE'] != 2) {
		// Previous button
		if ($page > 0) {
			$xml .= '<quad posn="4.95 0 0.12" sizen="3.2 3.2" action="'. $ast_config['MANIALINK_ID'] . ($page + 19) .'" style="Icons64x64_1" substyle="ArrowPrev"/>';
		}
		else {
			$xml .= '<quad posn="4.95 0 0.12" sizen="3.2 3.2" style="Icons64x64_1" substyle="StarGold"/>';
			$xml .= '<quad posn="4.95 0 0.13" sizen="3.2 3.2" style="Icons64x64_1" substyle="StarGold"/>';
		}

		// Next button (display only if more pages to display)
		if ( ($page < 15) && (count($ast_config['PLAYERS']) > 20) && (($page+1) < (ceil(count($ast_config['PLAYERS'])/20))) ) {
			$xml .= '<quad posn="8.25 0 0.12" sizen="3.2 3.2" action="'. $ast_config['MANIALINK_ID'] . ($page + 41) .'" style="Icons64x64_1" substyle="ArrowNext"/>';
		}
		else {
			$xml .= '<quad posn="8.25 0 0.12" sizen="3.2 3.2" style="Icons64x64_1" substyle="StarGold"/>';
			$xml .= '<quad posn="8.25 0 0.13" sizen="3.2 3.2" style="Icons64x64_1" substyle="StarGold"/>';
		}
	}

	$xml .= '</frame>';



	// BEGIN: Header Frame
	$xml .= '<frame posn="0 0 0">';

	if ($ast_config['CURRENT_GAMEMODE'] == 2) {
		// TeamMode has different Layout

		// BLUE: Avatar and Nickname of Player
		$xml .= '<quad posn="8.3 1.2 0.12" sizen="3.2 3.2" style="Icons64x64_1" substyle="Buddy"/>';
		$xml .= '<label posn="11.2 -0.15 0.12" sizen="15 3" halign="left" textsize="1" scale="0.7" text="$000PLAYERS TEAM BLUE"/>';

		// BLUE: Besttime (current) of Player
		$xml .= '<quad posn="29.6 0.8 0.12" sizen="2.3 2.3" style="Icons128x32_1" substyle="RT_TimeAttack"/>';
		$xml .= '<label posn="31.9 -0.15 0.12" sizen="5 3" halign="left" textsize="1" scale="0.7" text="$000TIME"/>';

		// BLUE: Personal Best of Player
		$xml .= '<quad posn="34.8 0.6 0.12" sizen="2.2 2.2" style="BgRaceScore2" substyle="ScoreLink"/>';
		$xml .= '<label posn="37.3 -0.15 0.12" sizen="5 3" halign="left" textsize="1" scale="0.7" text="$000BEST"/>';

		// RED: Avatar and Nickname of Player
		$xml .= '<quad posn="46.3 1.2 0.12" sizen="3.2 3.2" style="Icons64x64_1" substyle="Buddy"/>';
		$xml .= '<label posn="49.2 -0.15 0.12" sizen="15 3" halign="left" textsize="1" scale="0.7" text="$000PLAYERS TEAM RED"/>';

		// RED: Besttime (current) of Player
		$xml .= '<quad posn="67.6 0.8 0.12" sizen="2.3 2.3" style="Icons128x32_1" substyle="RT_TimeAttack"/>';
		$xml .= '<label posn="69.9 -0.15 0.12" sizen="5 3" halign="left" textsize="1" scale="0.7" text="$000TIME"/>';

		// RED: Personal Best of Player
		$xml .= '<quad posn="72.6 0.6 0.12" sizen="2.2 2.2" style="BgRaceScore2" substyle="ScoreLink"/>';
		$xml .= '<label posn="75.1 -0.15 0.12" sizen="5 3" halign="left" textsize="1" scale="0.7" text="$000BEST"/>';

	}
	else {
		// Default Layout

		// Avatar and Nickname of Player
		$xml .= '<quad posn="8.8 1.2 0.12" sizen="3.2 3.2" style="Icons64x64_1" substyle="Buddy"/>';
		$xml .= '<label posn="11.8 -0.15 0.12" sizen="5 3" halign="left" textsize="1" scale="0.7" text="$000PLAYERS"/>';

		if ( ($ast_config['CURRENT_GAMEMODE'] == 0) || ($ast_config['CURRENT_GAMEMODE'] == 3) || ($ast_config['CURRENT_GAMEMODE'] == 4) ) {
			// 0 = Rounds
			// 3 = Laps
			// 4 = Stunts
			// Besttime (current) of Player
			$xml .= '<quad posn="29.2 1.3 0.12" sizen="3.1 3.1" style="Icons128x128_1" substyle="Launch"/>';
			$xml .= '<label posn="32.2 -0.15 0.12" sizen="5 3" halign="left" textsize="1" scale="0.7" text="$000SCORE"/>';
		}
		else {
			// Besttime (current) of Player
			$xml .= '<quad posn="30.7 0.8 0.12" sizen="2.3 2.3" style="Icons128x32_1" substyle="RT_TimeAttack"/>';
			$xml .= '<label posn="33.1 -0.15 0.12" sizen="5 3" halign="left" textsize="1" scale="0.7" text="$000TIME"/>';
		}

		// Personal Best of Player
		$xml .= '<quad posn="36.8 0.6 0.12" sizen="2.2 2.2" style="BgRaceScore2" substyle="ScoreLink"/>';
		$xml .= '<label posn="39.3 -0.15 0.12" sizen="5 3" halign="left" textsize="1" scale="0.7" text="$000BEST"/>';

		// Score of Player
		$xml .= '<quad posn="42.45 0.8 0.12" sizen="1.8 2.4" style="BgRaceScore2" substyle="Points"/>';
		$xml .= '<label posn="44.5 -0.15 0.12" sizen="5 3" halign="left" textsize="1" scale="0.7" text="$000TOTAL"/>';

		// Times how often the finish has reached
		$xml .= '<quad posn="48.7 0.7 0.12" sizen="2.5 2.5" style="Icons128x128_1" substyle="Race"/>';
		$xml .= '<label posn="51.2 -0.15 0.12" sizen="5 3" halign="left" textsize="1" scale="0.7" text="$000FIN."/>';

		// Ladderrank of Player
		$xml .= '<quad posn="54.9 0.8 0.12" sizen="2.4 2.5" style="BgRaceScore2" substyle="LadderRank"/>';
		$xml .= '<label posn="57.25 -0.15 0.12" sizen="7 3" halign="left" textsize="1" scale="0.7" text="$000RANK"/>';

		// Ladderscore of Player
		$xml .= '<quad posn="61.2 0.7 0.12" sizen="2.4 2.4" style="Icons128x128_1" substyle="MedalCount"/>';
		$xml .= '<label posn="63.8 -0.15 0.12" sizen="7 3" halign="left" textsize="1" scale="0.7" text="$000POINTS"/>';

		// Online Since
		$xml .= '<quad posn="68.6 0.6 0.12" sizen="2.2 2.2" style="Icons64x64_1" substyle="StateFavourite"/>';
		$xml .= '<label posn="70.9 -0.15 0.12" sizen="7 3" halign="left" textsize="1" scale="0.7" text="$000ONLINE"/>';
	}

	// END: Header Frame
	$xml .= '</frame>';



	// BEGIN: Player lines
	$xml .= '<frame posn="2 -2 0">';

	// Create the columns
	if ($ast_config['CURRENT_GAMEMODE'] == 2) {
		// TeamMode has different Layout
		$xml .= '<quad posn="3.5 0 0.20" sizen="0.1 51.26" bgcolor="0003"/>';		// BLUE: Avatar and Nickname of Player
		$xml .= '<quad posn="27 0 0.20" sizen="0.1 51.26" bgcolor="0003"/>';		// BLUE: Besttime of Player
		$xml .= '<quad posn="32.4 0 0.20" sizen="0.1 51.26" bgcolor="0003"/>';		// BLUE: Personal Best of Player
		$xml .= '<quad posn="38 0 0.20" sizen="0.1 51.26" bgcolor="FFFF"/>';		// Team seperator
		$xml .= '<quad posn="41.5 0 0.20" sizen="0.1 51.26" bgcolor="0003"/>';		// RED: Avatar and Nickname of Player
		$xml .= '<quad posn="65 0 0.20" sizen="0.1 51.26" bgcolor="0003"/>';		// RED: Besttime of Player
		$xml .= '<quad posn="70.4 0 0.20" sizen="0.1 51.26" bgcolor="0003"/>';		// RED: Personal Best of Player

		// Build the Table content
		$xml .= ast_scoretableTeam($aseco, $caller);
	}
	else {
		// Default Layout
		$xml .= '<quad posn="3.9 0 0.20" sizen="0.1 51.26" bgcolor="0003"/>';		// Avatar and Nickname of Player
		$xml .= '<quad posn="27.2 0 0.20" sizen="0.1 51.26" bgcolor="0003"/>';		// Besttime of Player
		$xml .= '<quad posn="33.5 0 0.20" sizen="0.1 51.26" bgcolor="0003"/>';		// Personal Best of Player
		$xml .= '<quad posn="39.8 0 0.20" sizen="0.1 51.26" bgcolor="0003"/>';		// Score of Player
		$xml .= '<quad posn="45.9 0 0.20" sizen="0.1 51.26" bgcolor="0003"/>';		// Times how often the finish has reached
		$xml .= '<quad posn="51.1 0 0.20" sizen="0.1 51.26" bgcolor="0003"/>';		// Ladderrank of Player
		$xml .= '<quad posn="58.3 0 0.20" sizen="0.1 51.26" bgcolor="0003"/>';		// Ladderscore of Player
		$xml .= '<quad posn="65.5 0 0.20" sizen="0.1 51.26" bgcolor="0003"/>';		// Online Since
		$xml .= '<quad posn="72.5 0 0.20" sizen="0.1 51.26" bgcolor="0003"/>';		// Flag of Player

		// Build the Table content
		$xml .= ast_scoretableDefault($aseco, $caller, $page);
	}


	// END: Player lines
	$xml .= '</frame>';

	// END: Window Frame
	$xml .= '</frame>';

	$xml .= '</manialink>';


	// Disable automatic Scoretable
	setCustomUIField('scoretable', false);
	$xml .= getCustomUIBlock();

	$xml .= '</manialinks>';


	if ($caller == false) {
		// Send Scoretable to all connected Players
		$aseco->client->query('SendDisplayManialinkPage',$xml, ($timeout * 1000), false);

		// Set display status for all players
		foreach ($ast_config['PLAYERS'] as $login => $struct) {
			$ast_config['PLAYERS']{$login}->widget_status = true;
		}
	}
	else {
		// Send Scoretable to given Player ($caller = $player->login)
		$aseco->client->query('SendDisplayManialinkPageToLogin', $caller, $xml, ($timeout * 1000), false);

		// Set display status for this player
		$ast_config['PLAYERS']{$caller}->widget_status = true;
	}

}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function ast_scoretableTeam ($aseco, $caller = false) {
	global $ast_config;

	// Set line height for Player lines
	$line_height = 2.5;

	// Init $xml
	$xml = '';

	// Get the current Ranking from Server
	// The first parameter specifies the maximum number of infos to be returned, and the second one the starting index in the ranking
	$aseco->client->resetError();
	$aseco->client->query('GetCurrentRanking', 2,0);
	$ranks = $aseco->client->getResponse();

	if ( $aseco->client->isError() ) {
//		trigger_error('[plugin.alternate_scoretable.php] Error at GetCurrentRanking(): [' . $aseco->client->getErrorCode() . '] ' . $aseco->client->getErrorMessage(), E_USER_WARNING);
		return;
	}
	else {

		$xml .= '<frame posn="0 -2.5 0">';

		// Team Header Blue
		$xml .= '<quad posn="-7 0 0.05" sizen="5.8 5.8" style="Bgs1InRace" substyle="BgCard1"/>';
		$xml .= '<quad posn="-6.6 -0.15 0.06" sizen="5.4 5.48" bgcolor="03DF"/>';
		$xml .= '<label posn="-4.2 -1.2 0.07" sizen="3.5 3.5" halign="center" textsize="2" scale="1" text="$FFF'. $ranks[0]['Score'] .'"/>';
		$xml .= '<label posn="-4.2 -3.2 0.07" sizen="3.5 3.5" halign="center" textsize="1" scale="0.9" text="$FFF'. (($ranks[0]['Score'] == 1) ? 'Point' : 'Points') .'"/>';

		// Team Header Red
		$xml .= '<quad posn="77.2 0 0.05" sizen="5.8 5.8" style="Bgs1InRace" substyle="BgCard1"/>';
		$xml .= '<quad posn="77.1 -0.15 0.06" sizen="5.4 5.48" bgcolor="D30F"/>';
		$xml .= '<label posn="80.2 -1.2 0.07" sizen="3.5 3.5" halign="center" textsize="2" scale="1" text="$FFF'. $ranks[1]['Score'] .'"/>';
		$xml .= '<label posn="80.2 -3.2 0.07" sizen="3.5 3.5" halign="center" textsize="1" scale="0.9" text="$FFF'. (($ranks[1]['Score'] == 1) ? 'Point' : 'Points') .'"/>';

		$xml .= '</frame>';



		// Build the background lines to have full 20 team lines
		$position = 0;
		for ($i = 0; $i < 20; $i++) {
			$xml .= '<quad posn="0 '. $position .' 0.12" sizen="76 '. $line_height .'" image="'. $ast_config['URLS'][0]['BAR_TEAM'][0] .'"/>';
			$position -= ($line_height + 0.07);
		}

		$ranks = array();
		foreach ($ast_config['PLAYERS'] as $login => $struct) {
			if ($ast_config['PLAYERS']{$login}->finish_time == 0) {
				$ranks[$login] = 9999999999;			// For asort() need to != 0
			}
			else {
				$ranks[$login] = $ast_config['PLAYERS']{$login}->finish_time;
			}
		}
		asort($ranks, SORT_NUMERIC);


		$rank_count = 0;
		$team_offset = 0;
		for ($team = 0; $team <= 1; $team++) {

			// If Red Team, change offset
			if ($team == 1) {
				$team_offset = 38;
			}

			$position = 0;
			foreach ($ranks as $login => $finish_time) {

				// Work only on one team
				if ($ast_config['PLAYERS']{$login}->team != $team) {
					continue;
				}

				// Set Textcolor for connected/disconnected Players
				if ($ast_config['PLAYERS']{$login}->disconnected == true) {
					$text_color	= 'FFF9';
				}
				else {
					$text_color	= 'FFFF';
				}


				// Flag of Player
				$xml .= '<quad posn="'. ($team_offset + 1.8) .' '. $position .' 0.14" sizen="2.4 2.4" halign="center" image="tmtp://Skins/Avatars/Flags/'. (isset($ast_config['PLAYERS']{$login}->nation) ? $ast_config['PLAYERS']{$login}->nation : 'other') .'.dds"/>';

				// Avatar of Player (hide default TMN nation flags)
				if ( (isset($ast_config['PLAYERS']{$login}->avatar)) && (!preg_match('/Skins\/Avatars\/Flags\/(\w{3}|other)\.dds$/i', $ast_config['PLAYERS']{$login}->avatar)) ) {
					$xml .= '<quad posn="'. ($team_offset + 3.6) .' '. $position .' 0.14" sizen="2.5 2.5" bgcolor="000F" />';
					$xml .= '<quad posn="'. ($team_offset + 3.6) .' '. $position .' 0.15" sizen="2.5 2.5" image="tmtp://'. $ast_config['PLAYERS']{$login}->avatar .'"/>';
				}
				else {
					$xml .= '<quad posn="'. ($team_offset + 3.9) .' '. ($position - 0.25) .' 0.14" sizen="2.1 2.1" style="Icons128x128_1" substyle="Rankinks"/>';
				}

				// Nickname of Player
				if ($ast_config['PLAYERS']{$login}->disconnected != true) {
					// Link to PlayerWidget
					$xml .= '<quad posn="'. $team_offset .' '. $position .' 0.13" sizen="38 2.4" action="'. $ast_config['MANIALINK_ID'] . (60 + $ast_config['PLAYERS']{$login}->playerid) .'" bgcolor="FFF0"/>';	// Transparent background =D
				}
				$xml .= '<label posn="'. ($team_offset + 6.5) .' '. ($position - 0.3) .' 0.14" sizen="17.5 2.4" halign="left" textsize="1" scale="1" textcolor="'. $text_color .'" text="'. (isset($ast_config['PLAYERS']{$login}->nickname) ? $ast_config['PLAYERS']{$login}->nickname : '???') .'"/>';

				// Mark Player as Specator, if Player is Spectator
				if ($ast_config['PLAYERS']{$login}->isspectator == 1) {
					$xml .= '<quad posn="'. ($team_offset + 24.6) .' '. ($position - 0.3) .' 0.14" sizen="2 2" style="Icons64x64_1" substyle="Camera"/>';
				}

				// Besttime of Player ($caller = $player->login)
				if ( ($caller != false) && ($caller == $login) && ($ast_config['PLAYERS']{$login}->finish_time > 0) ) {
					$xml .= '<label posn="'. ($team_offset + 32) .' '. ($position - 0.6) .' 0.14" sizen="9 0" halign="right" style="TextTitle2Blink" textsize="1" scale="0.5" textcolor="'. $text_color .'" text="$W'. formatTime($ast_config['PLAYERS']{$login}->finish_time) .'"/>';
				}
				else {
					$xml .= '<label posn="'. ($team_offset + 31.9) .' '. ($position - 0.6) .' 0.14" sizen="8 0" halign="right" textsize="1" scale="0.9" textcolor="'. $text_color .'" text="'. formatTime($ast_config['PLAYERS']{$login}->finish_time) .'"/>';
				}

				// Personal Best of Player
				$xml .= '<label posn="'. ($team_offset + 37.5) .' '. ($position - 0.6) .' 0.14" sizen="5 2.4" halign="right" textsize="1" scale="0.9" textcolor="'. $text_color .'" text="'. (($ast_config['PLAYERS']{$login}->personal_best > 0) ? formatTime($ast_config['PLAYERS']{$login}->personal_best) : '--.--') .'"/>';

				$position -= ($line_height + 0.07);

				$rank_count += 1;

				// Not more then 20 entries per Page/Team
				if ($rank_count >= 20) {
					continue 1;
				}
			}
		}

		return $xml;
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function ast_scoretableDefault ($aseco, $caller = false, $page = 0) {
	global $ast_config;

	// Init $xml
	$xml = '';

	$devel = '';

	// Set line height for Player lines
	$line_height = 2.5;
	$position = 0;

	// Get the current Ranking from Server
	// The first parameter specifies the maximum number of infos to be returned, and the second one the starting index in the ranking
	$aseco->client->resetError();
	$aseco->client->query('GetCurrentRanking', 20,(($page * 20) ? ($page * 20) : 0));
	$ranks = $aseco->client->getResponse();

	if ( $aseco->client->isError() ) {
//		trigger_error('[plugin.alternate_scoretable.php] Error at GetCurrentRanking(): [' . $aseco->client->getErrorCode() . '] ' . $aseco->client->getErrorMessage(), E_USER_WARNING);
		return;
	}
	else {

		// Create the Players lines with content
		$rank_count = 0;
		foreach ($ranks as $pos) {

			// Set Textcolor for connected/disconnected Players
			if ($ast_config['PLAYERS']{$pos['Login']}->disconnected == true) {
				$text_color	= 'FFF9';
				$score_color	= 'FFA9';
			}
			else {
				$text_color	= 'FFFF';
				$score_color	= 'FFAF';
			}

			// Correct Ranking of GetCurrentRanking()
			$player_rank = (($page * 20) + $pos['Rank']);

			if ( ($pos['BestTime'] > 0) || ($pos['Score'] > 0) || ($pos['NbrLapsFinished'] > 0) ) {
				// Position of Player
				switch ($player_rank) {
					case 1:
						$xml .= '<quad posn="0 '. $position .' 0.12" sizen="76 '. $line_height .'" image="'. $ast_config['URLS'][0]['BAR_GOLD'][0] .'"/>';
						$xml .= '<quad posn="0.85 '. ($position - 0.1) .' 0.15" sizen="2.3 2.2" style="Icons64x64_1" substyle="First"/>';
						break;
					case 2:
						$xml .= '<quad posn="0 '. $position .' 0.12" sizen="76 '. $line_height .'" image="'. $ast_config['URLS'][0]['BAR_SILVER'][0] .'"/>';
						$xml .= '<quad posn="0.85 '. ($position - 0.1) .' 0.15" sizen="2.3 2.2" style="Icons64x64_1" substyle="Second"/>';
						break;
					case 3:
						$xml .= '<quad posn="0 '. $position .' 0.12" sizen="76 '. $line_height .'" image="'. $ast_config['URLS'][0]['BAR_BRONZE'][0] .'"/>';
						$xml .= '<quad posn="0.87 '. ($position - 0.1) .' 0.15" sizen="2.3 2.2" style="Icons64x64_1" substyle="Third"/>';
						break;
					default:
						$xml .= '<quad posn="0 '. $position .' 0.12" sizen="76 '. $line_height .'" image="'. $ast_config['URLS'][0]['BAR_DEFAULT'][0] .'"/>';
						$xml .= '<label posn="2 '. ($position - 0.5) .' 0.15" sizen="3 2.4" halign="center" textsize="1" scale="1" textcolor="'. $text_color .'" text="'. $player_rank .'"/>';
				}
			}
			else {
				$xml .= '<quad posn="0 '. $position .' 0.12" sizen="76 '. $line_height .'" image="'. $ast_config['URLS'][0]['BAR_NORANK'][0] .'"/>';
				$xml .= '<label posn="2 '. ($position - 0.5) .' 0.15" sizen="3 2.4" halign="center" textsize="1" scale="0.9" textcolor="'. $text_color .'" text="---"/>';
			}

			// Avatar of Player (hide default TMN nation flags)
			if ( (isset($ast_config['PLAYERS']{$pos['Login']}->avatar)) && (!preg_match('/Skins\/Avatars\/Flags\/(\w{3}|other)\.dds$/i', $ast_config['PLAYERS']{$pos['Login']}->avatar)) ) {
				$xml .= '<quad posn="4 '. $position .' 0.14" sizen="2.5 2.5" bgcolor="000F" />';
				$xml .= '<quad posn="4 '. $position .' 0.15" sizen="2.5 2.5" image="tmtp://'. $ast_config['PLAYERS']{$pos['Login']}->avatar .'"/>';
			}
			else {
				$xml .= '<quad posn="4.3 '. ($position - 0.25) .' 0.15" sizen="2.1 2.1" style="Icons128x128_1" substyle="Rankinks"/>';
			}

			// Nickname of Player
			if ($ast_config['PLAYERS']{$pos['Login']}->disconnected != true) {
				// Link to PlayerWidget
				$xml .= '<quad posn="0 '. $position .' 0.13" sizen="76 2.4" action="'. $ast_config['MANIALINK_ID'] . (60 + $ast_config['PLAYERS']{$pos['Login']}->playerid) .'" bgcolor="FFF0"/>';	// Transparent background =D
			}
			$xml .= '<label posn="6.9 '. ($position - 0.3) .' 0.14" sizen="17.5 2.4" halign="left" textsize="1" scale="1" textcolor="'. $text_color .'" text="'. (isset($ast_config['PLAYERS']{$pos['Login']}->nickname) ? $ast_config['PLAYERS']{$pos['Login']}->nickname : '???') .'"/>';

			// Mark Player as Specator, if Player is Spectator
			if ($ast_config['PLAYERS']{$pos['Login']}->isspectator == 1) {
				$xml .= '<quad posn="25 '. ($position - 0.3) .' 0.14" sizen="2 2" style="Icons64x64_1" substyle="Camera"/>';
			}


			if ( ($ast_config['CURRENT_GAMEMODE'] == 0) || ($ast_config['CURRENT_GAMEMODE'] == 3) || ($ast_config['CURRENT_GAMEMODE'] == 4) ) {
				// 0 = Rounds
				// 3 = Laps
				// 4 = Stunts
				// Score of Player ($caller = $player->login)
				if ( ($caller != false) && ($caller == $pos['Login']) && ($pos['BestTime'] > 0) ) {
					$xml .= '<label posn="33.2 '. ($position - 0.6) .' 0.14" sizen="9 0" halign="right" style="TextTitle2Blink" textsize="1" scale="0.5" textcolor="'. $text_color .'" text="$W'. (($pos['Score'] > 0) ? $pos['Score'] : '--') .'"/>';
				}
				else {
					$xml .= '<label posn="33.1 '. ($position - 0.6) .' 0.14" sizen="8 0" halign="right" textsize="1" scale="0.9" textcolor="'. $text_color .'" text="'. (($pos['Score'] > 0) ? $pos['Score'] : '--') .'"/>';
				}
			}
			else {
				// Besttime of Player ($caller = $player->login)
				if ( ($caller != false) && ($caller == $pos['Login']) && ($pos['BestTime'] > 0) ) {
					$xml .= '<label posn="33.2 '. ($position - 0.6) .' 0.14" sizen="9 0" halign="right" style="TextTitle2Blink" textsize="1" scale="0.5" textcolor="'. $text_color .'" text="$W'. formatTime($pos['BestTime']) .'"/>';
				}
				else {
					$xml .= '<label posn="33.1 '. ($position - 0.6) .' 0.14" sizen="8 0" halign="right" textsize="1" scale="0.9" textcolor="'. $text_color .'" text="'. formatTime($pos['BestTime']) .'"/>';
				}
			}

			// Personal Best of Player
			if ($ast_config['CURRENT_GAMEMODE'] == 4) {
				// Stunts
				$xml .= '<label posn="39.4 '. ($position - 0.6) .' 0.14" sizen="5 2.4" halign="right" textsize="1" scale="0.9" textcolor="'. $text_color .'" text="'. (($ast_config['PLAYERS']{$pos['Login']}->personal_best > 0) ? $ast_config['PLAYERS']{$pos['Login']}->personal_best : '--') .'"/>';
			}
			else {
				// All other Gamemodes
				$xml .= '<label posn="39.4 '. ($position - 0.6) .' 0.14" sizen="5 2.4" halign="right" textsize="1" scale="0.9" textcolor="'. $text_color .'" text="'. (($ast_config['PLAYERS']{$pos['Login']}->personal_best > 0) ? formatTime($ast_config['PLAYERS']{$pos['Login']}->personal_best) : '--') .'"/>';
			}

			// Total Score of Player
			$xml .= '<label posn="45.5 '. ($position - 0.6) .' 0.14" sizen="5.7 2.4" halign="right" textsize="1" scale="0.9" textcolor="'. $score_color .'" text="+'. sprintf("%.02f", $ast_config['PLAYERS']{$pos['Login']}->totalscore) .'"/>';

			if ($ast_config['CURRENT_GAMEMODE'] == 5) {
				// 5 = Cup
				// -> Score for this session
				$xml .= '<label posn="50.7 '. ($position - 0.6) .' 0.14" sizen="5 2.4" halign="right" textsize="1" scale="0.9" textcolor="'. $text_color .'" text="'. $pos['Score'] .'"/>';
			}
			else {
				// 0 = Rounds
				// 1 = TimeAttack
				// 2 = TeamMode (not handled here, seperate function)
				// 3 = Laps
				// 4 = Stunts
				// -> Times how often the finish was reached
				$xml .= '<label posn="50.7 '. ($position - 0.6) .' 0.14" sizen="5 2.4" halign="right" textsize="1" scale="0.9" textcolor="'. $text_color .'" text="'. number_format((int)$ast_config['PLAYERS']{$pos['Login']}->reached_finish, 0, '.', ' ') .'"/>';
			}

			// Ladderrank of Player
			$xml .= '<label posn="57.9 '. ($position - 0.6) .' 0.14" sizen="7 2.4" halign="right" textsize="1" scale="0.9" textcolor="'. $text_color .'" text="'. number_format((int)$ast_config['PLAYERS']{$pos['Login']}->ladderrank, 0, '.', ' ') .'"/>';

			// Ladderscore of Player
			$xml .= '<label posn="65.1 '. ($position - 0.6) .' 0.14" sizen="7 2.4" halign="right" textsize="1" scale="0.9" textcolor="'. $text_color .'" text="'. number_format((int)$ast_config['PLAYERS']{$pos['Login']}->ladderscore, 0, '.', ' ') .'"/>';

			// Online Since
			$xml .= '<label posn="72.2 '. ($position - 0.6) .' 0.14" sizen="7 2.4" halign="right" textsize="1" scale="0.9" textcolor="'. $text_color .'" text="'. (isset($ast_config['PLAYERS']{$pos['Login']}->online_since) ? $ast_config['PLAYERS']{$pos['Login']}->online_since : '--.--') .'"/>';

			// Flag of Player
			$xml .= '<quad posn="74.3 '. $position .' 0.14" sizen="2.4 2.4" halign="center" image="tmtp://Skins/Avatars/Flags/'. (isset($ast_config['PLAYERS']{$pos['Login']}->nation) ? $ast_config['PLAYERS']{$pos['Login']}->nation : 'other') .'.dds"/>';

			$position -= ($line_height + 0.07);

			$rank_count += 1;

			// Not more then 20 entries per Page
			if ($rank_count >= 20) {
				break;
			}

			$devel .= sprintf("%03d", $pos['Rank']) .' = '. $pos['Login'] ."\n";

			// Keep clean for each loop
			$pos = array();
		}

		// Fill empty line to have full 20 lines
		for ($i = $rank_count; $i < 20; $i++) {
			$xml .= '<quad posn="0 '. $position .' 0.12" sizen="76 '. $line_height .'" image="'. $ast_config['URLS'][0]['BAR_BLANK'][0] .'"/>';
			$position -= ($line_height + 0.07);
		}

		return $xml;
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function ast_widgetLegend ($aseco, $player, $display = true) {
	global $ast_config;

	$xml =  '<?xml version="1.0" encoding="UTF-8"?>';
	$xml .= '<manialinks id="'. $ast_config['MANIALINK_ID'] .'00">';
	$xml .= '<manialink id="'. $ast_config['MANIALINK_ID'] .'08"></manialink>';	// Turn of possible open Player-Window
	$xml .= '<manialink id="'. $ast_config['MANIALINK_ID'] .'04">';

	if ($display == true) {
		if ($aseco->server->gameinfo->mode == 2) {
			// TeamMode
			$xml .= '<frame posn="-4 -5.8 -2">';	// BEGIN: Window Frame
			$xml .= '<quad posn="0.8 -0.8 0.10" sizen="38.2 13.9" bgcolor="000A"/>';
			$xml .= '<quad posn="-0.2 0.2 0.11" sizen="40.2 15.9" style="Bgs1InRace" substyle="BgCard3"/>';
		}
		else {
			$xml .= '<frame posn="-4 11.2 -2">';	// BEGIN: Window Frame
			$xml .= '<quad posn="0.8 -0.8 0.10" sizen="38.2 30.9" bgcolor="000A"/>';
			$xml .= '<quad posn="-0.2 0.2 0.11" sizen="40.2 32.9" style="Bgs1InRace" substyle="BgCard3"/>';
		}


		// Close Button
		$xml .= '<frame posn="37.2 1.3 0">';
		$xml .= '<quad posn="0 0 0.12" sizen="4 4" style="Icons64x64_1" substyle="ArrowDown"/>';
		$xml .= '<quad posn="1.1 -1.35 0.13" sizen="1.8 1.75" bgcolor="EEEF"/>';
		$xml .= '<quad posn="0.65 -0.7 0.14" sizen="2.6 2.6" action="'. $ast_config['MANIALINK_ID'] .'06" style="Icons64x64_1" substyle="Close"/>';
		$xml .= '</frame>';


		// Frame for the Legend
		$xml .= '<frame posn="-6 -3.5 0">';

		// Avatar and Nickname of Player
		$xml .= '<quad posn="8.8 1.2 0.12" sizen="3.2 3.2" style="Icons64x64_1" substyle="Buddy"/>';
		$xml .= '<label posn="11.8 -0.15 0.12" sizen="5 3" halign="left" textsize="1" scale="0.8" text="$FFFPLAYERS"/>';
		$xml .= '<label posn="18 -0.05 0.12" sizen="35 3" halign="left" textsize="1" scale="0.9" text="$FFFThe Player Nicknames"/>';

		if ($aseco->server->gameinfo->mode == 0) {
			// 0 = Rounds
			$xml .= '<quad posn="8.2 -2.15 0.12" sizen="3.2 3.2" style="Icons128x128_1" substyle="Launch"/>';
			$xml .= '<label posn="11.8 -3.45 0.12" sizen="5 3" halign="left" textsize="1" scale="0.8" text="$FFFSCORE"/>';
			$xml .= '<label posn="18 -3.35 0.12" sizen="35 3" halign="left" textsize="1" scale="0.9" text="$FFFScore for this session"/>';
		}
		else if ( ($aseco->server->gameinfo->mode == 1) || ($aseco->server->gameinfo->mode == 2) || ($aseco->server->gameinfo->mode == 5) ) {
			// 1 = TimeAttack
			// 2 = Team
			// 5 = Cup
			$xml .= '<quad posn="8.8 -2.5 0.12" sizen="2.3 2.3" style="Icons128x32_1" substyle="RT_TimeAttack"/>';
			$xml .= '<label posn="11.8 -3.45 0.12" sizen="5 3" halign="left" textsize="1" scale="0.8" text="$FFFTIME"/>';
			$xml .= '<label posn="18 -3.35 0.12" sizen="35 3" halign="left" textsize="1" scale="0.9" text="$FFFYour current best time for this Track"/>';
		}
		else if ($aseco->server->gameinfo->mode == 3) {
			// 3 = Laps
			$xml .= '<quad posn="8.2 -2.15 0.12" sizen="3.2 3.2" style="Icons128x128_1" substyle="Launch"/>';
			$xml .= '<label posn="11.8 -3.45 0.12" sizen="5 3" halign="left" textsize="1" scale="0.8" text="$FFFSCORE"/>';
			$xml .= '<label posn="18 -3.35 0.12" sizen="35 3" halign="left" textsize="1" scale="0.9" text="$FFFCount how many checkpoints reached"/>';
		}
		else if ($aseco->server->gameinfo->mode == 4) {
			// 4 = Stunts
			$xml .= '<quad posn="8.2 -2.15 0.12" sizen="3.2 3.2" style="Icons128x128_1" substyle="Launch"/>';
			$xml .= '<label posn="11.8 -3.45 0.12" sizen="5 3" halign="left" textsize="1" scale="0.8" text="$FFFSCORE"/>';
			$xml .= '<label posn="18 -3.35 0.12" sizen="35 3" halign="left" textsize="1" scale="0.9" text="$FFFStunt-Points won for this session"/>';
		}

		// Personal Best of Player
		$xml .= '<quad posn="8.8 -6 0.12" sizen="2.2 2.2" style="BgRaceScore2" substyle="ScoreLink"/>';
		$xml .= '<label posn="11.8 -6.75 0.12" sizen="5 3" halign="left" textsize="1" scale="0.8" text="$FFFBEST"/>';
		$xml .= '<label posn="18 -6.65 0.12" sizen="35 3" halign="left" textsize="1" scale="0.9" text="$FFFYour personal best time for this Track"/>';

		if ($aseco->server->gameinfo->mode != 2) {
			// Total-Score of Player
			$xml .= '<quad posn="8.8 -9.5 0.12" sizen="1.8 2.4" style="BgRaceScore2" substyle="Points"/>';
			$xml .= '<label posn="11.8 -10.25 0.12" sizen="5 3" halign="left" textsize="1" scale="0.8" text="$FFFTOTAL"/>';
			$xml .= '<label posn="18 -10.15 0.12" sizen="35 3" halign="left" textsize="1" scale="0.9" text="$FFFYour Total-Score at this Server and this Session"/>';

			// Finish count
			$xml .= '<quad posn="8.8 -13 0.12" sizen="2.5 2.5" style="Icons128x128_1" substyle="Race"/>';
			$xml .= '<label posn="11.8 -13.75 0.12" sizen="5 3" halign="left" textsize="1" scale="0.8" text="$FFFFIN."/>';
			if ($aseco->server->gameinfo->mode == 5) {
				// Cup: Score for this session
				$xml .= '<label posn="18 -13.65 0.12" sizen="35 3" halign="left" textsize="1" scale="0.9" text="$FFFYour current Score"/>';
			}
			else {
				$xml .= '<label posn="18 -13.65 0.12" sizen="35 3" halign="left" textsize="1" scale="0.9" text="$FFFCount how often you´ve finished this Track"/>';
			}

			// Ladderrank of Player
			$xml .= '<quad posn="8.8 -16.5 0.12" sizen="2.4 2.5" style="BgRaceScore2" substyle="LadderRank"/>';
			$xml .= '<label posn="11.8 -17.25 0.12" sizen="7 3" halign="left" textsize="1" scale="0.8" text="$FFFRANK"/>';
			$xml .= '<label posn="18 -17.15 0.12" sizen="35 3" halign="left" textsize="1" scale="0.9" text="$FFFYour current Ladder-Rank"/>';

			// Ladderscore of Player
			$xml .= '<quad posn="8.8 -20 0.12" sizen="2.4 2.4" style="Icons128x128_1" substyle="MedalCount"/>';
			$xml .= '<label posn="11.8 -20.75 0.12" sizen="7 3" halign="left" textsize="1" scale="0.8" text="$FFFPOINTS"/>';
			$xml .= '<label posn="18 -20.65 0.12" sizen="35 3" halign="left" textsize="1" scale="0.9" text="$FFFYour current Ladder-Score"/>';

			// Online Since
			$xml .= '<quad posn="8.8 -23.5 0.12" sizen="2.2 2.2" style="Icons64x64_1" substyle="StateFavourite"/>';
			$xml .= '<label posn="11.8 -24.25 0.12" sizen="7 3" halign="left" textsize="1" scale="0.8" text="$FFFONLINE"/>';
			$xml .= '<label posn="18 -24.15 0.12" sizen="35 3" halign="left" textsize="1" scale="0.9" text="$FFFYour Online-Time since connect to this Server"/>';
		}
		$xml .= '</frame>';


		$xml .= '</frame>';	// END: Window Frame
	}

	$xml .= '</manialink>';
	$xml .= '</manialinks>';

	$aseco->client->addCall('SendDisplayManialinkPageToLogin', array($player->login, $xml, 0, false));
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function ast_widgetPlayer ($aseco, $target, $caller) {
	global $ast_config;

	// Bail out if Player is disconnected
	if ($ast_config['PLAYERS']{$target->login}->disconnected == true) {
		return;
	}

	$xml =  '<?xml version="1.0" encoding="UTF-8"?>';
	$xml .= '<manialinks id="'. $ast_config['MANIALINK_ID'] .'00">';
	$xml .= '<manialink id="'. $ast_config['MANIALINK_ID'] .'04"></manialink>';	// Turn of possible open Legend-Window
	$xml .= '<manialink id="'. $ast_config['MANIALINK_ID'] .'08">';

		if ($ast_config['CURRENT_GAMEMODE'] == 2) {
			if ($ast_config['PLAYERS']{$target->login}->team == 0) {
				// Team Blue Position
				$xml .= '<frame posn="-4 20 -1">';	// BEGIN: Window Frame
			}
			else {
				// Team Red Position
				$xml .= '<frame posn="-36 20 -1">';	// BEGIN: Window Frame
			}
		}
		else {
			// Default position
			$xml .= '<frame posn="-4 20 -1">';	// BEGIN: Window Frame
		}
		$xml .= '<quad posn="0.8 -0.8 0.10" sizen="38.2 39.7" bgcolor="000A"/>';
		$xml .= '<quad posn="-0.2 0.2 0.11" sizen="40.2 41.7" style="Bgs1InRace" substyle="BgCard3"/>';

		// Close Button
		$xml .= '<frame posn="37.2 1.3 0">';
		$xml .= '<quad posn="0 0 0.12" sizen="4 4" style="Icons64x64_1" substyle="ArrowDown"/>';
		$xml .= '<quad posn="1.1 -1.35 0.13" sizen="1.8 1.75" bgcolor="EEEF"/>';
		$xml .= '<quad posn="0.65 -0.7 0.14" sizen="2.6 2.6" action="'. $ast_config['MANIALINK_ID'] .'09" style="Icons64x64_1" substyle="Close"/>';
		$xml .= '</frame>';

		// Frame for content
		$xml .= '<frame posn="2.5 -3.5 0">';

		// Avatar of Player (hide default TMN nation flags)
		$xml .= '<quad posn="0 1 0.12" sizen="12.8 12.8" bgcolor="000F"/>';
		if ( (isset($ast_config['PLAYERS']{$target->login}->avatar)) && (!preg_match('/Skins\/Avatars\/Flags\/(\w{3}|other)\.dds$/i', $ast_config['PLAYERS']{$target->login}->avatar)) ) {
			$xml .= '<quad posn="0 1 0.14" sizen="12.8 12.8" image="tmtp://'. $ast_config['PLAYERS']{$target->login}->avatar .'"/>';
		}
		else {
			$xml .= '<quad posn="0 1 0.13" sizen="12.8 12.8" style="Icons128x128_1" substyle="Rankinks"/>';
		}

		// Player Login/Nickname
		$xml .= '<label posn="13.8 1 0.12" sizen="21 3" halign="left" textsize="2" scale="0.8" text="$FFF$S'. ast_handleSpecialChars($target->nickname) .'"/>';
		$xml .= '<label posn="13.8 -1.6 0.12" sizen="21 1.5" halign="left" textsize="1" scale="0.9" text="$FFF'. $target->login .'"/>';

		// Display Team Membership of Player
		if ($ast_config['CURRENT_GAMEMODE'] == 2) {
			if ($ast_config['PLAYERS']{$target->login}->team == 0) {
				$xml .= '<label posn="13.8 -4 0.12" sizen="21 1.5" halign="left" textsize="1" scale="0.9" text="$09FMember of Team Blue"/>';
			}
			else {
				$xml .= '<label posn="13.8 -4 0.12" sizen="21 1.5" halign="left" textsize="1" scale="0.9" text="$F20Member of Team Red"/>';
			}
		}


		// Is Player Driver or Spectator?
		if ($ast_config['PLAYERS']{$target->login}->isspectator == 1) {
			$xml .= '<quad posn="29.4 -3.6 0.12" sizen="3.4 3.4" style="Icons64x64_1" substyle="CameraLocal"/>';
			$xml .= '<label posn="30.9 -6.8 0.12" sizen="12 1.6" halign="center" textsize="1" scale="0.75" text="$FFFSpectator"/>';
		}
		else {
			$xml .= '<quad posn="29.4 -3.6 0.12" sizen="3.4 3.4" style="Icons128x128_1" substyle="Vehicles"/>';
			$xml .= '<label posn="30.9 -6.8 0.12" sizen="12 1.6" halign="center" textsize="1" scale="0.75" text="$FFFPlayer"/>';
		}

		// Is Player Operator, Admin or MasterAdmin?
		if ( $aseco->isMasterAdminL($target->login) ) {
			$xml .= '<quad posn="29.4 -8.8 0.12" sizen="3.4 3.4" style="Icons128x128_1" substyle="Invite"/>';
			$xml .= '<label posn="30.9 -12 0.12" sizen="12 1.6" halign="center" textsize="1" scale="0.75" text="$FFFMasterAdmin"/>';
		}
		else if ( $aseco->isAdminL($target->login) ) {
			$xml .= '<quad posn="29.4 -8.8 0.12" sizen="3.4 3.4" style="Icons128x128_1" substyle="Rankinks"/>';
			$xml .= '<label posn="30.9 -12 0.12" sizen="12 1.6" halign="center" textsize="1" scale="0.75" text="$FFFAdmin"/>';
		}
		else if ( $aseco->isOperatorL($target->login) ) {
			$xml .= '<quad posn="29.4 -8.8 0.12" sizen="3.4 3.4" style="Icons128x128_1" substyle="ChallengeAuthor"/>';
			$xml .= '<label posn="30.9 -12 0.12" sizen="12 1.6" halign="center" textsize="1" scale="0.75" text="$FFFOperator"/>';
		}

//		// Add Server to Favorites
//		$xml .= '<quad posn="29.4 -8.8 0.12" sizen="3.4 3.4" url="tmtp://#addfavourite='. $aseco->server->serverlogin .'" style="Icons128x128_1" substyle="ServersFavorites"/>';
//		$xml .= '<label posn="30.9 -12 0.12" sizen="12 1.6" url="tmtp://#addfavourite='. $aseco->server->serverlogin .'" halign="center" textsize="1" scale="0.75" text="$FFFServer to Favorites"/>';
//
//		// Add Player to Buddylist
//		if ($target->login != $caller->login) {
//			$xml .= '<quad posn="29.4 -14 0.12" sizen="3.4 3.4" url="tmtp://#addbuddy='. $target->login .'" style="Icons128x128_1" substyle="Solo"/>';
//			$xml .= '<label posn="30.9 -17.2 0.12" sizen="12 1.6" url="tmtp://#addbuddy='. $target->login .'" halign="center" textsize="1" scale="0.75" text="$FFFAdd to Buddylist"/>';
//		}


		// LastMatchScore of Player
		$xml .= '<label posn="0 -13.5 0.12" sizen="8 1.5" halign="left" textsize="1" scale="0.9" text="$FFFLast Score"/>';
		$xml .= '<label posn="9 -13.5 0.12" sizen="25.5 1.5" halign="left" textsize="1" scale="0.8" text="$CC8$W+'. sprintf("%.02f", $ast_config['PLAYERS']{$target->login}->lastscore) .'"/>';

		// Ladderrank of Player
		$xml .= '<label posn="0 -15.5 0.12" sizen="8 1.5" halign="left" textsize="1" scale="0.9" text="$FFFLadder-Rank"/>';
		$xml .= '<label posn="9 -15.5 0.12" sizen="25.5 1.5" halign="left" textsize="1" scale="0.9" text="$FFF'. number_format((int)$target->ladderrank, 0, '.', ' ') .'"/>';

		// Ladderscore of Player
		$xml .= '<label posn="0 -17.5 0.12" sizen="8 1.5" halign="left" textsize="1" scale="0.9" text="$FFFLadder-Score"/>';
		$xml .= '<label posn="9 -17.5 0.12" sizen="25.5 1.5" halign="left" textsize="1" scale="0.9" text="$FFF'. number_format((int)$target->ladderscore, 0, '.', ' ') .'"/>';


		// NbrMatchWins of Player
		$xml .= '<label posn="0 -20.5 0.12" sizen="8 1.5" halign="left" textsize="1" scale="0.9" text="$FFFTotal Won"/>';
		$xml .= '<label posn="9 -20.5 0.12" sizen="25.5 1.5" halign="left" textsize="1" scale="0.9" text="$FFF'.  number_format($ast_config['PLAYERS']{$target->login}->match_wins, 0, '.', ' ') .'"/>';

		// NbrMatchDraws of Player
		$xml .= '<label posn="0 -22.5 0.12" sizen="8 1.5" halign="left" textsize="1" scale="0.9" text="$FFFTotal Draws"/>';
		$xml .= '<label posn="9 -22.5 0.12" sizen="25.5 1.5" halign="left" textsize="1" scale="0.9" text="$FFF'. number_format($ast_config['PLAYERS']{$target->login}->match_draws, 0, '.', ' ') .'"/>';

		// NbrMatchLosses of Player
		$xml .= '<label posn="0 -24.5 0.12" sizen="8 1.5" halign="left" textsize="1" scale="0.9" text="$FFFTotal Losses"/>';
		$xml .= '<label posn="9 -24.5 0.12" sizen="25.5 1.5" halign="left" textsize="1" scale="0.9" text="$FFF'. number_format($ast_config['PLAYERS']{$target->login}->match_losses, 0, '.', ' ') .'"/>';


		// Zone from Player
		$xml .= '<label posn="0 -27.5 0.12" sizen="8 1.5" halign="left" textsize="1" scale="0.9" text="$FFFZone"/>';
		$xml .= '<quad posn="9 -27 0.12" sizen="2.4 2.4" halign="left" image="tmtp://Skins/Avatars/Flags/'. $ast_config['PLAYERS']{$target->login}->nation .'.dds"/>';
		$xml .= '<label posn="12 -27.5 0.12" sizen="25.5 1.5" halign="left" textsize="1" scale="0.9" text="$FFF'. str_replace('|', ', ', $target->zone) .'"/>';

		// OnlineRights of Player
		$xml .= '<label posn="0 -29.5 0.12" sizen="8 1.5" halign="left" textsize="1" scale="0.9" text="$FFFGame"/>';
		$xml .= '<label posn="9 -29.5 0.12" sizen="25.5 1.5" halign="left" textsize="1" scale="0.9" text="$FFF'. $ast_config['PLAYERS']{$target->login}->game_rights .'"/>';

		// ClientVersion of Player
		$xml .= '<label posn="0 -31.5 0.12" sizen="8 1.5" halign="left" textsize="1" scale="0.9" text="$FFFClient"/>';
		$xml .= '<label posn="9 -31.5 0.12" sizen="25.5 1.5" halign="left" textsize="1" scale="0.9" text="$FFF'. $ast_config['PLAYERS']{$target->login}->client_version .'"/>';

		// Online Since
		$xml .= '<label posn="0 -33.5 0.12" sizen="8 1.5" halign="left" textsize="1" scale="0.9" text="$FFFOnline"/>';
		$xml .= '<label posn="9 -33.5 0.12" sizen="25.5 1.5" halign="left" textsize="1" scale="0.9" text="$FFF'. $ast_config['PLAYERS']{$target->login}->online_since .'"/>';

		$xml .= '</frame>';	// END: Frame for content

		$xml .= '</frame>';	// END: Window Frame

	$xml .= '</manialink>';
	$xml .= '</manialinks>';

	// Send Quicklink to given Player ($caller = $player)
	$aseco->client->addCall('SendDisplayManialinkPageToLogin', array($caller->login, $xml, 0, false));
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function ast_widgetQuicklink ($aseco, $caller = false, $display = true) {
	global $ast_config;


	$xml =  '<?xml version="1.0" encoding="UTF-8"?>';
	$xml .= '<manialinks id="'. $ast_config['MANIALINK_ID'] .'00">';
	$xml .= '<manialink id="'. $ast_config['MANIALINK_ID'] .'07">';

	if ($display == true) {
		$xml .= '<frame posn="'. $ast_config['QUICKLINK_BUTTON'][0]['WIDGET_POSITION_X'][0] .' '. $ast_config['QUICKLINK_BUTTON'][0]['WIDGET_POSITION_Y'][0] .' 0">';
		$xml .= '<quad posn="0 0 10" sizen="7 4" action="'. $ast_config['MANIALINK_ID'] .'02" style="Icons64x64_1" substyle="RestartRace"/>';
		$xml .= '</frame>';
	}

	$xml .= '</manialink>';
	$xml .= '</manialinks>';

	if ($caller == false) {
		// Send Quicklink to all connected Players
		$aseco->client->query('SendDisplayManialinkPage', $xml, 0, false);
	}
	else {
		// Send Quicklink to given Player ($caller = $player)
		$aseco->client->query('SendDisplayManialinkPageToLogin', $caller->login, $xml, 0, false);
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onPlayerConnect
function ast_onPlayerConnect ($aseco, $player) {
	global $ast_config;


	// Get the detailed Player informations
	$aseco->client->resetError();
	$aseco->client->query('GetDetailedPlayerInfo', $player->login);
	$info = $aseco->client->getResponse();

	if ( !$aseco->client->isError() ) {
		$ast_config['PLAYERS']{$player->login}->avatar = $info['Avatar']['FileName'];
		$ast_config['PLAYERS']{$player->login}->game_rights = (($info['OnlineRights'] == 3) ? 'United' : 'Nations');
		$ast_config['PLAYERS']{$player->login}->client_version = $info['ClientVersion'];

		$ast_config['PLAYERS']{$player->login}->match_wins = $info['LadderStats']['NbrMatchWins'];
		$ast_config['PLAYERS']{$player->login}->match_draws = $info['LadderStats']['NbrMatchDraws'];
		$ast_config['PLAYERS']{$player->login}->match_losses = $info['LadderStats']['NbrMatchLosses'];
		$ast_config['PLAYERS']{$player->login}->team = $info['TeamId'];
	}

	$ast_config['PLAYERS']{$player->login}->playerid = $player->id;

	$ast_config['PLAYERS']{$player->login}->ladderscore = $player->ladderscore;
	$ast_config['PLAYERS']{$player->login}->ladderrank = (($player->ladderrank != -1) ? $player->ladderrank : 0);
	$ast_config['PLAYERS']{$player->login}->personal_best = $player->panels['pb'];
	$ast_config['PLAYERS']{$player->login}->nation = mapCountry($player->nation);

	$ast_config['PLAYERS']{$player->login}->reached_finish = 0;
	$ast_config['PLAYERS']{$player->login}->totalscore = 0;
	$ast_config['PLAYERS']{$player->login}->lastscore = 0;
	$ast_config['PLAYERS']{$player->login}->lastscore_overall = array();
	$ast_config['PLAYERS']{$player->login}->finish_time = 0;

	$ast_config['PLAYERS']{$player->login}->isspectator = $player->isspectator;
	$ast_config['PLAYERS']{$player->login}->disconnected = false;
	$ast_config['PLAYERS']{$player->login}->widget_status = false;
	$ast_config['PLAYERS']{$player->login}->delayed_call = false;

	// Remove all $S from Nickname and set at first a $S for shadows in Scoretable
	$ast_config['PLAYERS']{$player->login}->nickname = '$S'. ast_handleSpecialChars($player->nickname);

	// If nation is 'OTH' rename to 'other' to get the right flag (other.dds)
	if ( ($ast_config['PLAYERS']{$player->login}->nation == 'OTH') || ($ast_config['PLAYERS']{$player->login}->nation == '') ) {
		$ast_config['PLAYERS']{$player->login}->nation = 'other';
	}


	// Show the Quicklink button?
	if ( (strtoupper($ast_config['QUICKLINK_BUTTON'][0]['ENABLED'][0]) == 'TRUE') && ($ast_config['CURRENT_STATE'] != 6) ) {
		// $aseco, $caller, $display
		ast_widgetQuicklink($aseco, $player, true);
	}

	// Close Scoretable and set CustomUI (main intention here)
	$aseco->client->addCall('SendDisplayManialinkPageToLogin', array($player->login, ast_closeAllWidgets($aseco, true, false), 0, false));
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onPlayerFinish
function ast_onPlayerFinish ($aseco, $finish_item) {
	global $ast_config;


	// Close all Scoretable-Widgets, if open
	if ($finish_item->score == 0) {
		if ( ($ast_config['PLAYERS']{$finish_item->player->login}->widget_status == true) || ($ast_config['PLAYERS']{$finish_item->player->login}->delayed_call != false) ) {
			// Close Scoretable and possible open LegendWidget/PlayerWidget
			$aseco->client->query('SendDisplayManialinkPageToLogin', $finish_item->player->login, ast_closeAllWidgets($aseco, false, false), 0, false);

			$ast_config['PLAYERS']{$finish_item->player->login}->delayed_call = false;
			$ast_config['PLAYERS']{$finish_item->player->login}->widget_status = false;
		}
		return;
	}

	// Count the finishline crossings
	$ast_config['PLAYERS']{$finish_item->player->login}->reached_finish += 1;

	// Save the current finish time in TeamMode
	if ($ast_config['CURRENT_GAMEMODE'] == 2) {
		$ast_config['PLAYERS']{$finish_item->player->login}->finish_time = $finish_item->score;
	}

	// Do not display Scoretable on Multilap-Maps
	if (strtoupper($ast_config['SHOW_AT_FINISH'][0]) == 'TRUE') {

		// No Scoretable at Multilap Maps and Gamemode 'Rounds' and 'Team' at Player finish
		if ( ($finish_item->challenge->laprace == false) && ($ast_config['CURRENT_GAMEMODE'] != 0) && ($ast_config['CURRENT_GAMEMODE'] != 2) ) {

			$ast_config['PLAYERS']{$finish_item->player->login}->delayed_call = array(
				'delay'		=> (time() + $ast_config['DELAY_FINISH_DISPLAY'][0]),
				'func'		=> 'ast_showScoretable($aseco, "'. $finish_item->player->login .'", 0, true, 0);',
			);
		}
	}

}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onPlayerInfoChanged
function ast_onPlayerInfoChanged ($aseco, $info) {
	global $ast_config;

	// Find out the TeamId
	$ast_config['PLAYERS']{$info['Login']}->team = $info['TeamId'];
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onPlayerDisconnect
function ast_onPlayerDisconnect ($aseco, $player) {
	global $ast_config;


	$ast_config['PLAYERS']{$player->login}->disconnected = true;
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// $answer = [0]=PlayerUid, [1]=Login, [2]=Answer
// called @ onPlayerManialinkPageAnswer
function ast_onPlayerManialinkPageAnswer ($aseco, $answer) {
	global $ast_config;


	// If id = 0, bail out immediately
	if ($answer[2] == 0) {
		return;
	}

	// Get Player
	$player = $aseco->server->players->getPlayer($answer[1]);

	if ($answer[2] == $ast_config['MANIALINK_ID'] .'02') {			// Display Scoretable

		// $aseco, $caller, $timeout, $display_close, $page
		ast_showScoretable($aseco, $player->login, 0, true, 0);

	}
	else if ($answer[2] == $ast_config['MANIALINK_ID'] .'03') {		// Close Scoretable and possible open LegendWidget/PlayerWidget

		$aseco->client->addCall('SendDisplayManialinkPageToLogin', array($player->login, ast_closeAllWidgets($aseco, false, false), 0, false));

	}
	else if ($answer[2] == $ast_config['MANIALINK_ID'] .'05') {		// Display Legend

		ast_widgetLegend($aseco, $player, true);

	}
	else if ($answer[2] == $ast_config['MANIALINK_ID'] .'06') {		// Close Legend

		ast_widgetLegend($aseco, $player, false);

	}
	else if ($answer[2] == $ast_config['MANIALINK_ID'] .'09') {		// Close Player

		$xml =  '<?xml version="1.0" encoding="UTF-8"?>';
		$xml .= '<manialinks id="'. $ast_config['MANIALINK_ID'] .'00">';
		$xml .= '<manialink id="'. $ast_config['MANIALINK_ID'] .'08"></manialink>';	// PlayerWidget
		$xml .= '</manialinks>';
		$aseco->client->addCall('SendDisplayManialinkPageToLogin', array($player->login, $xml, 0, false));

	}
	else if ( ($answer[2] >= (int)$ast_config['MANIALINK_ID'] .'20') && ($answer[2] <= (int)$ast_config['MANIALINK_ID'] .'35') ) {		// Previous pages

		// Get the wished Page
		$page = intval( str_replace($ast_config['MANIALINK_ID'], '', $answer[2]) - 20 );

		// $aseco, $caller, $timeout, $display_close, $page
		ast_showScoretable($aseco, $player->login, 0, true, $page);

	}
	else if ( ($answer[2] >= (int)$ast_config['MANIALINK_ID'] .'40') && ($answer[2] <= (int)$ast_config['MANIALINK_ID'] .'55') ) {		// Next pages

	// Get the wished Page
		$page = intval( str_replace($ast_config['MANIALINK_ID'], '', $answer[2]) - 40 );

		// $aseco, $caller, $timeout, $display_close, $page
		ast_showScoretable($aseco, $player->login, 0, true, $page);

	}
	else if ( ($answer[2] >= sprintf("%d%d", $ast_config['MANIALINK_ID'], 60)) && ($answer[2] <= sprintf("%d%d", $ast_config['MANIALINK_ID'], 9999999)) ) {

		// Remove the MANIALINK_ID(+60) from target PlayerID
		$PlayerID = str_replace($ast_config['MANIALINK_ID'], '', $answer[2]);
		$PlayerID -= 60;

		foreach ($aseco->server->players->player_list as &$target) {
			if ($target->id == $PlayerID) {
				ast_widgetPlayer($aseco, $target, $player);
				break;
			}
		}

	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onEndRace1
function ast_onEndRace1 ($aseco, $race) {
	global $ast_config;


	// Hide the Quicklink button on 'onEndRace' event
	if (strtoupper($ast_config['QUICKLINK_BUTTON'][0]['ENABLED'][0]) == 'TRUE') {
		// $aseco, $caller, $display
		ast_widgetQuicklink($aseco, false, false);
	}

	// Disable all (ast)Widgets and the automatic Scoretable
	$aseco->client->query('SendDisplayManialinkPage', ast_closeAllWidgets($aseco, false, true), 0, false);
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onEndRace
function ast_onEndRace ($aseco, $race) {
	global $ast_config, $ast_todo;


	// Set current state
	$ast_config['CURRENT_STATE'] = 6;

	// ast_showScoretable($aseco, $caller, $timeout, $display_close, $page)
	$ast_todo[] = array(
		'delay'		=> (time() + $ast_config['DELAY_SCORE_DISPLAY'][0]),
		'func'		=> 'ast_showScoretable($aseco, false, 0, false, 0);',
	);
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onRestartChallenge2
function ast_onRestartChallenge2 ($aseco, $challenge) {
	global $ast_config, $ast_todo;


	// Set current State
	$ast_config['CURRENT_STATE'] = 1;

	// Set current Gamemode
	$ast_config['CURRENT_GAMEMODE'] = $aseco->server->gameinfo->mode;

	// Show the Quicklink button?
	if (strtoupper($ast_config['QUICKLINK_BUTTON'][0]['ENABLED'][0]) == 'TRUE') {
		// $aseco, $caller, $display
		ast_widgetQuicklink($aseco, false, true);
	}

	// Reset Todo-List
	$ast_todo = array();

	// Close the Alternate-Scoretable at all players
	$aseco->client->query('SendDisplayManialinkPage', ast_closeAllWidgets($aseco, false, false), 0, false);
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onNewChallenge
function ast_onNewChallenge ($aseco, $race) {
	global $ast_config, $ast_todo;


	$aseco->client->query('ManualFlowControlEnable', true);


	// Reset Todo-List
	$ast_todo = array();

	// Close the alternate Scoretable at all players
	$aseco->client->query('SendDisplayManialinkPage', ast_closeAllWidgets($aseco, false, false), 0, false);

	// Set current State
	$ast_config['CURRENT_STATE'] = 1;

	// Set current Gamemode
	$ast_config['CURRENT_GAMEMODE'] = $aseco->server->gameinfo->mode;

	// Show the Quicklink button?
	if (strtoupper($ast_config['QUICKLINK_BUTTON'][0]['ENABLED'][0]) == 'TRUE') {
		// $aseco, $caller, $display
		ast_widgetQuicklink($aseco, false, true);
	}


	// Remove all disconnected players
	foreach ($ast_config['PLAYERS'] as $login => $struct) {
		if ($ast_config['PLAYERS']{$login}->disconnected == true) {
			unset($ast_config['PLAYERS']{$login});
		}
	}
//	// Remove all disconnected players (alternate)
//	$tmp['PLAYERS'] = array();
//	foreach ($ast_config['PLAYERS'] as $login => $struct) {
//		if ($ast_config['PLAYERS']{$login}->disconnected == false) {
//			$tmp['PLAYERS']{$login} = $struct;
//		}
//	}
//	$ast_config['PLAYERS'] = $tmp['PLAYERS'];



	// Refresh Player status
	foreach ($aseco->server->players->player_list as $player) {

		// Refresh totalscore of each (connected) Player
		$aseco->client->resetError();
		$aseco->client->query('GetDetailedPlayerInfo', $player->login);
		$info = $aseco->client->getResponse();

		if ( !$aseco->client->isError() ) {

			// If the Player did not get a score at the last match, then $info['LadderStats']['LastMatchScore']
			// has the score from the last scored match inside. Need to reset it here, otherwise the ->totalscore
			// grows wrong.
			if ( !in_array($info['LadderStats']['LastMatchScore'], $ast_config['PLAYERS']{$player->login}->lastscore_overall) ) {
				$ast_config['PLAYERS']{$player->login}->lastscore = $info['LadderStats']['LastMatchScore'];

				// Add won LadderPoints
				$ast_config['PLAYERS']{$player->login}->lastscore_overall[] = $info['LadderStats']['LastMatchScore'];

				// Safe mem.
				$ast_config['PLAYERS']{$player->login}->lastscore_overall = array_unique($ast_config['PLAYERS']{$player->login}->lastscore_overall);
			}
			else {
				$ast_config['PLAYERS']{$player->login}->lastscore = 0;
			}
			$ast_config['PLAYERS']{$player->login}->totalscore += $ast_config['PLAYERS']{$player->login}->lastscore;

			// Get actual LadderStats
			$ast_config['PLAYERS']{$player->login}->match_wins = $info['LadderStats']['NbrMatchWins'];
			$ast_config['PLAYERS']{$player->login}->match_draws = $info['LadderStats']['NbrMatchDraws'];
			$ast_config['PLAYERS']{$player->login}->match_losses = $info['LadderStats']['NbrMatchLosses'];
		}

		// Reset finishline crossings
		$ast_config['PLAYERS']{$player->login}->reached_finish = 0;

		// Reset finish time
		$ast_config['PLAYERS']{$player->login}->finish_time = 0;

		// Get actual Ladder of Player
		$ast_config['PLAYERS']{$player->login}->ladderscore = $player->ladderscore;
		$ast_config['PLAYERS']{$player->login}->ladderrank = (($player->ladderrank != -1) ? $player->ladderrank : 0);
	}

	$aseco->client->query('ManualFlowControlEnable', false);
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onBeginRound
function ast_onBeginRound ($aseco) {

	// Disable all (ast)Widgets
	$aseco->client->query('SendDisplayManialinkPage', ast_closeAllWidgets($aseco, false, false), 0, false);
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onEndRound
function ast_onEndRound ($aseco) {
	global $ast_config, $ast_todo;


	// Do not display Scoretable on Multilap-Maps
	if (strtoupper($ast_config['SHOW_AT_FINISH'][0]) == 'TRUE') {

		// Display Scoretable at Gamemode 'Rounds' and 'TeamMode' at EndRound
		if ( ($ast_config['CURRENT_GAMEMODE'] == 0) || ($ast_config['CURRENT_GAMEMODE'] == 2) ) {
			// ast_showScoretable($aseco, $caller, $timeout, $display_close, $page)
			ast_showScoretable($aseco, false, 0, true, 0);
		}
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function ast_closeAllWidgets ($aseco, $preload = false, $hide_automatic = false) {
	global $ast_config;

	$xml =  '<?xml version="1.0" encoding="UTF-8"?>';
	$xml .= '<manialinks id="'. $ast_config['MANIALINK_ID'] .'00">';
	$xml .= '<manialink id="'. $ast_config['MANIALINK_ID'] .'01">';

	if ($preload == true) {
		// Place preload outside visible area
		$xml .= '<frame posn="-600 -600 0">';
		$xml .= '<quad posn="0 0 0" sizen="76 2.5" image="'. $ast_config['URLS'][0]['BAR_GOLD'][0] .'"/>';
		$xml .= '<quad posn="0 0 0" sizen="76 2.5" image="'. $ast_config['URLS'][0]['BAR_SILVER'][0] .'"/>';
		$xml .= '<quad posn="0 0 0" sizen="76 2.5" image="'. $ast_config['URLS'][0]['BAR_BRONZE'][0] .'"/>';
		$xml .= '<quad posn="0 0 0" sizen="76 2.5" image="'. $ast_config['URLS'][0]['BAR_DEFAULT'][0] .'"/>';
		$xml .= '<quad posn="0 0 0" sizen="76 2.5" image="'. $ast_config['URLS'][0]['BAR_NORANK'][0] .'"/>';
		$xml .= '<quad posn="0 0 0" sizen="76 2.5" image="'. $ast_config['URLS'][0]['BAR_BLANK'][0] .'"/>';
		$xml .= '<quad posn="0 0 0" sizen="76 2.5" image="'. $ast_config['URLS'][0]['BAR_TEAM'][0] .'"/>';
		$xml .= '</frame>';
	}

	$xml .= '</manialink>';	// Scoretable
	$xml .= '<manialink id="'. $ast_config['MANIALINK_ID'] .'04"></manialink>';	// Legend
	$xml .= '<manialink id="'. $ast_config['MANIALINK_ID'] .'08"></manialink>';	// PlayerWidget


	if ( (strtoupper($ast_config['SHOW_AT_FINISH'][0]) == 'TRUE') || ($hide_automatic == true) ) {
		// Disable automatic Scoretable
		setCustomUIField('scoretable', false);
	}
	else {
		// Enable automatic Scoretable
		setCustomUIField('scoretable', true);
	}
	$xml .= getCustomUIBlock();

	$xml .= '</manialinks>';

	return $xml;
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function ast_handleSpecialChars ($string) {


	// Remove links, e.g. "$(L|H)[...]...$(L|H)"
	$string = preg_replace('/\${1}(L|H)\[.*?\](.*?)\$(L|H)/i', '$2', $string);
	$string = preg_replace('/\${1}(L|H)\[.*?\](.*?)/i', '$2', $string);

	// Remove $S (shadow)
	// Remove $H (manialink)
	// Remove $W (wide)
	// Remove $I (italic)
	// Remove $L (link)
	// Remove $O (bold)
	// Remove $N (narrow)
	$string = preg_replace('/\${1}[SHWILON]/i', '', $string);

	// Convert &
	// Convert "
	// Convert '
	// Convert >
	// Convert <
	$string = str_replace(
			array(
				'&',
				'"',
				"'",
				'>',
				'<'
			),
			array(
				'&amp;',
				'&quot;',
				'&apos;',
				'&gt;',
				'&lt;'
			),
			$string
	);

	return validateUTF8String($string);	// validateUTF8String() from basic.inc.php
}

?>
